package net.sourceforge.jFuzzyLogic;

import java.io.ByteArrayInputStream;
import java.util.HashMap;

import net.sourceforge.jFuzzyLogic.defuzzifier.DefuzzifierCenterOfGravity;
import net.sourceforge.jFuzzyLogic.membership.MembershipFunction;
import net.sourceforge.jFuzzyLogic.membership.MembershipFunctionTriangular;
import net.sourceforge.jFuzzyLogic.membership.Value;
import net.sourceforge.jFuzzyLogic.rule.LinguisticTerm;
import net.sourceforge.jFuzzyLogic.rule.Rule;
import net.sourceforge.jFuzzyLogic.rule.RuleBlock;
import net.sourceforge.jFuzzyLogic.rule.Variable;

public class ZzzExample {

	public static void main(String args[]) {
		FIS fis = new FIS();

		FunctionBlock fb = new FunctionBlock(fis);

		Variable v = new Variable("a", 0.0, 20.0);
		MembershipFunction triangular = new MembershipFunctionTriangular(new Value(0.0000343232), new Value(10.0), new Value(13.0));

		LinguisticTerm t1 = new LinguisticTerm("lt1", triangular);
		LinguisticTerm t2 = new LinguisticTerm("lt2", triangular);

		v.add(t1);
		v.add(t2);

		Variable v_out = new Variable("out", 0.0, 20.0);
		MembershipFunction triangularOut = new MembershipFunctionTriangular(new Value(2.0), new Value(10.0), new Value(13.0));
		v_out.setDefaultValue(2.0);
		v_out.setDefuzzifier(new DefuzzifierCenterOfGravity(v_out));

		fb.setVariable("a", v);
		fb.setVariable("out", v_out);

		v_out.add(t1);
		v_out.add(t2);

		RuleBlock rules = new RuleBlock(fb);
		rules.setName("No1");

		Rule r = new Rule("Rule1", rules);

		r.addAntecedent(v, "lt1", false);
		r.addConsequent(v_out, "lt2", false);

		rules.add(r);

		HashMap<String, RuleBlock> ruleBlocksMap = new HashMap<String, RuleBlock>();

		ruleBlocksMap.put(rules.getName(), rules);

		fb.setRuleBlocks(ruleBlocksMap);

		// Print, save and load
		System.out.println(fb.toString());
		ByteArrayInputStream iStr = new ByteArrayInputStream(fb.toString().getBytes());
		FIS.load(iStr, true);
	}
}
